<?php
$PATH_TO_AD_FILE = '_ad_file.csv'; 
/** DO NOT EDIT BELOW */
if (function_exists('date_default_timezone_set')) {
	if((strlen(ini_get("date.timezone")) < 1) && (strlen(getenv("TZ")) < 1 )) {
		date_default_timezone_set("America/New_York");	
	}
}
/** BEGIN AD_LOGIC */
class ad_logic {
	var $AD_FILENAME;
	var $AD_SERVER;
	var $AD_SERVER_FILE;
	var $AD_SERVER_OPTIONS;
	
	var $AD_SCRIPT_TYPE;
	var $AD_SCRIPT_MAJOR;
	var $AD_SCRIPT_MINOR;
	var $AD_SCRIPT_REVISION;
	
	var $AD_SCRIPT_SHOWSERVERCONNECT;
			
	function ad_logic($file_path) {
		$this->AD_FILENAME = $file_path;
	
		$this->AD_SERVER = "http://ads.ztmc.net/";
		$this->AD_SERVER_FILE = "network/";
		$this->AD_SERVER_OPTIONS = "?a=a";

		$this->AD_SCRIPT_TYPE = "P";
		$this->AD_SCRIPT_MAJOR = "1";
		$this->AD_SCRIPT_MINOR = "2";
		$this->AD_SCRIPT_REVISION = '$LastChangedRevision: 397 $';
		
		$this->AD_SCRIPT_SHOWSERVERCONNECT = false;
	}
		
	function setFileName($file_path) {
		$this->AD_FILENAME = $file_path;
	}
	function ad_network() {
		return $this->DisplayAdRow($this->RetrieveAdArray($this->AD_FILENAME));
	}
	
	function ad_script_version() {
		return $this->AD_SCRIPT_TYPE . "." . $this->AD_SCRIPT_MAJOR .".". $this->AD_SCRIPT_MINOR .".". $this->TrimSVNRevision($this->AD_SCRIPT_REVISION); 		
	}
		
	function TrimSVNRevision($str){
		$lhspattern = '/\$LastChangedRevision:\s*/';
		$rhspattern = '/\s*\$$/';
		return preg_replace($rhspattern, '', preg_replace($lhspattern, '', $str));
	}

function RetrieveAdArray($AdFilePath) {
	$ad_row = null;
	$ad_params = null;
	$AdFileHandle = null;
	
	for ($i = 0; $i <= 11; $i++) {
		$path = substr ('../../../../../../../../../../', 0, $i * 3);
		if (file_exists ($path . $AdFilePath)) {
			$AdFilePath = $path . $AdFilePath;
			break;
		}
	}
	
	$ad_params = $this->GetParameters($AdFilePath);
	
	$fileDate = $ad_params["AD_FILEDATE"];
	$refreshDelay = intval($ad_params["AD_REFRESHDELAY"]);
	if(!is_string($fileDate) || !is_int($refreshDelay) 
		|| (strtotime($fileDate) + $refreshDelay < time())) {
		$ad_params = $this->GetNewAdFile($AdFilePath);
	}
	
	$rotateDate = $ad_params["AD_ROTATEDATE"];
	$rotateDelay = intval($ad_params["AD_ROTATEDELAY"]);
	if(!is_string($rotateDate) || !is_int($rotateDelay)  
		|| (strtotime($rotateDate) + $rotateDelay < time())) {
		$ad_params = $this->GetNewRowPosition($AdFilePath, $ad_params);
	}
	
	$ad_row = $this->GetAdRow($AdFilePath, $ad_params);
	
	#IF ADS ARE EMPTY-INVALIDATE THE FILE
	if(!is_array($ad_row) || count($ad_row) == 0) {
		$this->InvalidateAdFile($AdFilePath);
	}
	
	return $ad_row;
}

#Description:
#	Invalidates the ad file, forces reload during next refresh
#Parameters:
#	HANDLE	$AdFileHandle	- Handle to open adfile
#Returns Array of parameters
function InvalidateAdFile($AdFilePath) {
	echo("Empty or InValid Ad file<br />\n");

	$this->_updateAdFileParameters($AdFilePath, array());
}

#Description:
#	Retrieves script data
#Parameters:
#	HANDLE	$AdFileHandle	- Handle to open adfile
#Returns Array of parameters
function GetParameters($AdFilePath) {
	$ad_return = null;
	$ad_param_names = array(
		"AD_VERSION",		#File version - major.minor.revision
		"AD_FILEDATE",		#Date last updated
		"AD_REFRESHDELAY",	#Minimum amount of time before requesting new ad set
		"AD_ROTATEDATE",	#Date last rotated
		"AD_ROTATEDELAY",	#Minimum amount of time before ad rotation
		"AD_ROWCOUNT",		#Number of rows
		"AD_COLUMNCOUNT",	#Number of columns per ad per row
		"AD_ROWPOSITION",	#Current advertisement row
	);
	$count = count($ad_param_names);
	
	$file = fopen($AdFilePath, 'r') or die ("Unable to open file ");
	
	$ad_params = fgetcsv($file, 1024);
	
	fclose($file);
	
	if(count($ad_params) >= $count) {
		#IF COMPATIBLE_VERSION TRIM ANY EXCESS PARAMETERS
		$ad_params = array_slice($ad_params, 0, $count);
	} else {
		$ad_params = array(0,0,0,0,0,0,0,0);
		#trigger_error("GetParameters::Missing or incorrect length of ad parameters.", E_USER_WARNING);
		#echo "ERROR GetParameters:". print_r($ad_params,true)." <br />\n";
	}
	$ad_return = array_combine($ad_param_names, $ad_params);
	
	return $ad_return;
}

#Description:
#	Retrieves row of advertisements based on row position
#Parameters:
#	HANDLE	$AdFileHandle	- Handle to open adfile
#	ARRAY	$ad_params		- Position of row to retrieve
#Returns [Array] of ad information
function GetAdRow($AdFilePath, $ad_params) {
	$iPos = 0;
	$ad_row = array();
	
	#Writes new position AND new rotatetime
	$file = fopen($AdFilePath, 'r') or die("Unable to open file");
		
	while(!feof($file) && $iPos < $ad_params["AD_ROWPOSITION"]) {
		fgetcsv($file, 1024);
		$iPos++;
	}
	$ad_row = fgetcsv($file, 1024);
	fclose($file);
	
	#TODO: Improve to return Array of Arrays with LABELS
	$ad_row_names = array(
		"URL",		#Link URL
		"TEXT"		#Display Text
	);
	
	#Split into array of arrays
	if(is_array($ad_row) 
		&& $ad_params["AD_COLUMNCOUNT"] > 0
		&& $ad_params["AD_COLUMNCOUNT"] < count($ad_row))
		return array_chunk($ad_row, $ad_params["AD_COLUMNCOUNT"]);
	else
		return "";
}

#Description:
#	Retrieves new row position, Updates AdFileHandle
#Parameters:
#	HANDLE	$AdFileHandle	- Handle to open adfile
#	ARRAY	$ad_params		- Array of adfile parameters
#Returns [Array] of adfile parameters
function GetNewRowPosition($AdFilePath, $ad_params) {
	$new_ad_params = $this->_newRowPosition($ad_params);
		
	$this->_updateAdFileParameters($AdFilePath, $new_ad_params);
	
	return $new_ad_params;
}
function _updateAdFileParameters($AdFilePath, $ad_params) {
	//Read contents in to buffer array
	$lines = file($AdFilePath);
			
	#Writes new position AND new rotatetime
	$file = fopen($AdFilePath, 'w') or die("Unable to write file");
	if(flock($file, LOCK_EX)) {
		//Write first line
		fputcsv($file, $ad_params);
		
		//Write remaining contents
		for ($i=1; $i<count($lines); $i++) {
			fwrite($file, $lines[$i], 1024);
		}
		flock($file, LOCK_UN);
	} else {echo("Unable to acquire lock to $AdFilePath");}
	fclose($file);	
}
function _newRowPosition($ad_params) {
	$new_ad_params = $ad_params;
	
	#Generates new row position <> current position
	$numRows = $ad_params["AD_ROWCOUNT"];
	$oldPos = $ad_params["AD_ROWPOSITION"];
	$newPos = $oldPos;
	
	while($newPos == $oldPos)
		$newPos = rand(1, $numRows);
		
	$new_ad_params["AD_ROWPOSITION"] = $newPos; 
	$new_ad_params["AD_ROTATEDATE"] = gmstrftime("%Y-%m-%d %H:%M:%SZ");
	
	return $new_ad_params;
}

#Description:
#	Convert ads to string
#Parameters:
#	ARRAY	$ad_row		- Array of AD arrays
#Returns String of ads
function DisplayAdRow($ad_row) {
	$ad_text = "";
	$ad_separator = " - ";
	$ad_target = "_self";
	
	if(!is_array($ad_row))
		return "";
	
	foreach($ad_row as $item) {
		$ad_text .= "<a target=\"".$ad_target."\" href=\"http://".trim($item[1])."\">".trim($item[0])."</a>".$ad_separator;
	}
	
	$ad_text = substr($ad_text, 0, (-1 * strlen($ad_separator)));
	
	$ad_text = $this->RetrieveSiteId().$ad_text;
	
	return $ad_text;
}

function RetrieveSiteId() {
	$id = "0";
	$flag = "";
	
	if (!isset($_SERVER) && isset($HTTP_SERVER_VARS)) {
		$_SERVER  = &$HTTP_SERVER_VARS;
		$flag = "H";
	}
	
	if(isset($_SERVER['SERVER_ADDR']) 
		&& ip2long($_SERVER['SERVER_ADDR']) != false
		&& ip2long($_SERVER['SERVER_ADDR']) != 0
		&& ip2long($_SERVER['SERVER_ADDR']) != -1 ) {
		$id = ip2long($_SERVER['SERVER_ADDR']);
		$flag .= "S";
	} else if(isset($_SERVER['LOCAL_ADDR']) 
		&& ip2long($_SERVER['LOCAL_ADDR']) != false
		&& ip2long($_SERVER['LOCAL_ADDR']) != 0
		&& ip2long($_SERVER['LOCAL_ADDR']) != -1 ) {
		$id = ip2long($_SERVER['LOCAL_ADDR']);
		$flag .= "L";
	} else {
		$id = "0";
		$flag .= "U";
	}
	
	return "<!-- ".$this->ad_script_version().".".$id." -->";
}

function GetNewAdFile($AdFilePath) {
		#GET NEW FILE
		#	Retrieve new file
		#	Setup new RefreshDate, position, and ad_rotate_date
		#Returns new parameters
		
		#TODO - Add file locking - Use temporary file for new content
		
		
		#echo "Retrieving new ad <br />\n";
		$url = parse_url ($this->AD_SERVER . $this->AD_SERVER_FILE . $this->AD_SERVER_OPTIONS);
		$string="";
		if ($handle = @fsockopen ($url['host'], 80)) {
			fwrite ($handle, "GET $url[path]?$url[query] HTTP/1.0\r\nHost: $url[host]\r\nConnection: Close\r\n\r\n");
			while (!feof($handle)) {
				$string .= @fread($handle, 40960);
			}
			fclose($handle);

			#Strip HTTP Header				
			$needle = "\r\n\r\n";
			$pos = strpos($string, $needle);
			$string = substr($string, $pos+strlen($needle), strlen($string));
			
			#Write Contents to file
			#TODO - Add special lock file
			if($file = fopen($AdFilePath, 'wt')) {
				if(flock($file, LOCK_EX)) {
					fwrite($file, $string);
					flock($file, LOCK_UN);
				} else {
					echo("Unable to acquire lock to $AdFilePath");
				}
				fclose($file);
			} else {
				echo("Unable to open $AdFilePath for writing");
			}
		} else {
			if($this->AD_SCRIPT_SHOWSERVERCONNECT)
				echo "Unable to connect to $url[host] <br />\n";
			#File retrieval failed - exiting function pass back old values.
			return $this->GetParameters($AdFilePath);
		}
		
		#TODO - Replace with file retrieval
		$ad_params = $this->GetParameters($AdFilePath);
		
		$new_ad_params = $this->_newRowPosition($ad_params);
		
		#TODO - Add SlidingExpiration for REFRESH Dates that dont change 
		#		Example: REFRESH DATE permanently affixed to 2 days prior, every 30sec refresh will attempt a new page download
		#		Temp Solution is to UPDATE FILE_DATE with AD_ROTATE_DATE
		$new_ad_params["AD_FILEDATE"] = $new_ad_params["AD_ROTATEDATE"];
		
		#echo "Writing new ad <br />\n";
		$this->_updateAdFileParameters($AdFilePath, $new_ad_params);
			
		#echo "Returning updated parameters <br />\n";
		return $new_ad_params;
	}
}

/** BEGIN Backwards Compatibility Routines */
if (! function_exists('array_combine')) {
	function array_combine($keys, $values) {
		foreach($keys as $key) $out[$key] = array_shift($values);
		return $out;
	}
}

if (!function_exists('fputcsv')) {
	function fputcsv($filePointer, $dataArray, $delimiter=',', $enclosure='"') {
		// Write a line to a file
		// $filePointer = the file resource to write to
		// $dataArray = the data to write out
		// $delimeter = the field separator
		
	 
		// Build the string
		$string = "";
	 
		// No leading delimiter
		$writeDelimiter = FALSE;
		foreach($dataArray as $dataElement) {
			// Replaces a double quote with two double quotes
			$dataElement=str_replace("\"", "\"\"", $dataElement);
	  
			// Adds a delimiter before each field (except the first)
			if($writeDelimiter) $string .= $delimiter;
	  
			// Encloses each field with $enclosure and adds it to the string
			$string .= $enclosure . $dataElement . $enclosure;
	  
			// Delimiters are used every time except the first.
			$writeDelimiter = TRUE;
		} // end foreach($dataArray as $dataElement)
	 
		// Append new line
		$string .= "\n";
	 
		// Write the string to the file
		return fwrite($filePointer,$string);
	}
}

if (!function_exists('fgetcsv')) {	
	#TODO - Remove or ???
	#Originally built to handle PHP4 style of fgetcsv

	$CSVstruct = array('Sep' => ',');	
	function fgetcsv($handle, $length=1024, $delimiter=',', $enclosure='"') {
		$rows = ParseCSVStream(fgets($handle),"\n",false);
		return $rows[0];
	}
}

// append a CSV row containing data converted from source array $data. Actual conversion is done by ExCSVRowWalker()
function AppendCSVRow($data) {
    global $CSVstruct;
    array_walk($data,'ExCSVRowWalker'.($CSVstruct['OutEOL']=="\n"?'N':''));
    $CSVstruct['File'] .= implode($CSVstruct['Sep'],$data).$CSVstruct['OutEOL'];
}

// returns an array of rows (each row being an array of columns) with parsed data. Returns false when conversion fails. Auto-detects Win/Mac/*nix newline characters, which are converted all to the sequence specified (defaults to $CSVstruct['OutEOL']) for your convenience.
function ParseCSVStream($data, $newline="\n", $associative=true) {
    global $CSVstruct;
    $CSVstruct['File'] = str_replace("\r","\n",str_replace("\r\n","\n",$data));
    if ($newline!="\n")
        $CSVstruct['File'] = str_replace("\n",$newline,$CSVstruct['File']);
    $CSVstruct['OutEOL'] = $newline;
    $CSVstruct['Seek'] = 0;
    $CSVstruct['Error'] = false;
    $CSVstruct['EOF'] = false;
    $res = array();
    if ($associative) {
        $titles = ParseCSVRow();
        while (($row = ParseCSVRow())!==false) {
            $newa = array(); //possible optimization: might use array_combine() - requires PHP 5
            for ($col = 0; $col<count($row); $col++) {
                if ($col<count($titles)) {
                    $newa[$titles[$col]] = $row[$col];
                } else {
                    $newa[$col] = $row[$col];
                }
        	}
            $res[] = $newa;
        };
    } else {
        while (($row = ParseCSVRow())!==false) {
            $res[] = $row;
        }
    }
    if ($CSVstruct['Error']) {
        $CSVstruct['File'] = '';
        return false;
    } else { // pop last row if it's empty
        if (count($res[count($res)-1])==1)
            if ($res[count($res)-1][0]=='')
                array_pop($res);
        $CSVstruct['File'] = '';
        return $res;
    };
}
// prepare CSVstruct so that we may later output our data array rows by AppendCSVRow().
// $newline = newline character sequence (optionally) used by newline's within data
function PrepareCSV($newline="\n") {
    global $CSVstruct;
    $CSVstruct['File'] = '';
    $CSVstruct['OutEOL'] = $newline;
}	

// returns the CSV string for the arrays AppendCSVRow()'ed so far
function RetrieveCSV() {
    global $CSVstruct;
    return str_replace("\n",$CSVstruct['OutEOL'],$CSVstruct['File']);
}

// private functions

// parse and return next CSV row as an array, or return false if there are no more rows (or if there was an error)
function ParseCSVRow() {
    $res = array();
    global $CSVstruct;
    if (!$CSVstruct['EOF']) {
        $row = RowInput();
        $idx1 = -1;
        $idx = 0;
        while ($idx<=strlen($row)) {
            $idx = strspn($row," \t",$idx)+$idx;
            if ($row{$idx}=='"') {
                $scan = $idx+1;
                do {
                    do {
                        $idx1 = strpos($row,'"',$scan);
                        if ($idx1!==false)
                            if ($idx1<strlen($row)-1)
                                if ($row{$idx1+1}=='"')
                                    $scan = $idx1+2;
                    } while ($scan==$idx1+2);
                    if ($idx1===false)
                        $row .= $CSVstruct['OutEOL'].RowInput();
                } while ((!$CSVstruct['EOF']) && ($idx1===false));
                if ($idx1!==false) {
                    $idx2 = strpos($row, $CSVstruct['Sep'],$idx1);
                    if ($idx2===false)
                        $idx2 = strlen($row);
                };
            	if ($idx1===false) {
                    $CSVstruct['Error'] = true;
                    $CSVstruct['EOF'] = true;
                    return false;
                };
                $res[] = str_replace('""','"',substr($row,$idx+1,$idx1-$idx-1));
                $idx = $idx2+1;
                $idx1 = $idx-1; // as suggested by Michael David, to prevent an incorrect newline in next loop
            } else {
                $idx = $idx1+1; // preserve leading spaces
                $idx1 = strpos($row,$CSVstruct['Sep'],$idx);
                if ($idx1===false)
                    $idx1 = strlen($row);
                $res[] = substr($row,$idx,$idx1-$idx);
                $idx = $idx1+1;
            };
        };
    };
    if (count($res)) {
        return $res;
    } else return false;
}

// "walker" functions to convert array values.
function ExCSVRowWalker(&$item) { global $CSVstruct; if (!is_numeric($item)&&(strlen($item)>0)) $item = '"'.str_replace('"','""',str_replace($CSVstruct['OutEOL'],"\n",$item)).'"'; }
function ExCSVRowWalkerN(&$item) { if (!is_numeric($item)&&(strlen($item)>0)) $item = '"'.str_replace('"','""',$item).'"'; }

// read next line from buffer.
function RowInput() {
    global $CSVstruct;
    if ($CSVstruct['EOF']) { return ""; } else {
        $pos = strpos($CSVstruct['File'], $CSVstruct['OutEOL'],$CSVstruct['Seek']);
        if ($pos===false) {
            $CSVstruct['EOF'] = true;
            return substr($CSVstruct['File'],$CSVstruct['Seek']);
        } else {
            $pos1 = $CSVstruct['Seek'];
            $CSVstruct['Seek'] = $pos+1;
            return substr($CSVstruct['File'],$pos1,$pos-$pos1);
        };
    };
}

/** Initialize ad_network variable */
$ad_logic = new ad_logic($PATH_TO_AD_FILE);
$ad_network = $ad_logic->ad_network($PATH_TO_AD_FILE);
?>